/*	Interface to INFLATE.C decompression routines / tar stream handler
	Copyright (c) 1996 Tillmann Steinbrecher */


/* #define DEBUG	*/

#ifndef PROG_NAME
	#define	PROG_NAME	"gzip-engine"
#endif


/* gzip header bit flag definitions */
#define IS_MULTI	2
#define IS_EXTRA	4
#define IS_FILENAME	8
#define	IS_COMMENT	16
#define IS_ENCRYPT	32

/* pkzip header bit flag definitions */

#define PK_IS_ENCRYPT 1
#define PK_IS_ENDHEADER 9
#define PK_IS_EXTENDED 8

UWORD mask_bits[] = {
    0x0000,
    0x0001, 0x0003, 0x0007, 0x000f, 0x001f, 0x003f, 0x007f, 0x00ff,
    0x01ff, 0x03ff, 0x07ff, 0x0fff, 0x1fff, 0x3fff, 0x7fff, 0xffff
};

int outcnt=0;
ULONG outsiz=0;
byte *outptr;
byte *outbuf;
FILE* in;
char headertype;

typedef struct{
	UWORD magic;
	byte method;
	byte flags;
	ULONG time;
	byte extra_flags;
	byte os;
}gzip_hdr;

typedef struct{
        unsigned long magic;
	unsigned char version;
	unsigned char os;
	unsigned int bitflag;
	unsigned int method;
	unsigned int modtime;
	unsigned int moddate;
	unsigned long crc32;
	unsigned long c_size;
	unsigned long u_size;
	unsigned int filename_size;
	unsigned int extra_field_length;
}pkzip_hdr;


typedef struct{
	ULONG crc32;
	ULONG size;
}gzip_end;

typedef struct{
	long magic;		/* NO magic number according to APPNOTE.TXT,
				   but to ensure compability with Tar, we add
				   it ... uuuh... */
        unsigned long crc32;
        unsigned long comp_size;
        unsigned long size;
}pkzip_end;



pkzip_hdr pk_header;
gzip_hdr gz_header;

int check_valid(void){
	gzip_end end_info;
        pkzip_end pk_end_info;
        ULONG new_crc32, storedcrc;
	new_crc32=updcrc(outbuf,0L);
switch(headertype){
case HEADER_GZIP:
        fread(&end_info,sizeof(end_info),1,in);
	storedcrc=end_info.crc32;
	break;
case HEADER_PKZIP:
        storedcrc=pk_header.crc32;
	break;
case HEADER_PKZIP_END:
        fread(&pk_end_info,sizeof(pk_end_info),1,in);
        storedcrc=pk_end_info.crc32;
	break;
default:
        printf(PROG_NAME ": internal error - cannot locate crc32\n");
        return -1;
}


  if(new_crc32!=storedcrc){
		printf(PROG_NAME ": warning - broken file(s), crc32 error!\n");
		return -1;
	       }
	else if(command==CMD_TEST||command==CMD_LIST) printf("\rArchive has valid crc32.\n");
	return 0;
 }

int read_tar_header(void){
	  TARHEADER unc_hdr;
	  long csum;
	  fread(&unc_hdr,512,1,in);
	  sscanf(unc_hdr.ocsum, "%lo", &csum);
	  if(istarheader((char*)&unc_hdr,csum,0)){
		fseek(in,0,SEEK_SET);
		printf("Archive is in uncompressed TAR format\n");
		return 0;
		}
	    else return 1;
	  }


int read_gzip_header(void){

	char orig_name[260]="";/* 260 bytes hopefully is enough for filename */
	char comments[4096];   /* 4096 bytes hopefully is enough for comments */
	char ch;
	struct time filetime;
	struct date filedate;

        fread(&gz_header,sizeof(gz_header),1,in);

         if(gz_header.magic!=0x8b1f) {
/*        printf(PROG_NAME ": warning - bad magic number (0x%x), not a gzip file.\n",gz_header.magic); */
	  return 1;
	 }

        headertype=HEADER_GZIP;

        if(gz_header.method!=8) {
		printf(PROG_NAME ": error - unknown compression (%i)\n",gz_header.method);
		return -1;
		}
	if(gz_header.flags & IS_MULTI) {
		fprintf(stderr, PROG_NAME ": error - multi-volume archives not supported\n");
		return -1;
		}
        if(gz_header.flags & IS_EXTRA){
		fprintf(stderr, PROG_NAME ": error - don't know how to handle extended gzip header.\n");
		return -1;
		}
        if(gz_header.flags & IS_FILENAME){
		fscanf(in,"%s",orig_name);
		if(getc(in)==EOF) {
			fprintf(stderr, PROG_NAME ": error - unexpected EOF\n");
			return -1;
		       }
		}
        if(gz_header.flags & IS_COMMENT){
		fscanf(in,"%s",comments);
		fprintf(stderr, "comments:\n%s",comments);
		if(getc(in)==EOF) {
			fprintf(stderr, PROG_NAME ": error - unexpected EOF\n");
			return -1;
		       }

		}
	if(gz_header.flags & IS_ENCRYPT){
		fprintf(stderr, PROG_NAME ": error - file is encrypted.\n");
		return -1;
		}

    #ifdef TIME_BUG_WORKAROUND

        #ifdef _OS2_VERSION_                            /* OS/2 Version */
                if(timezone==18000) gz_header.time -= 10800;
                else gz_header.time= gz_header.time - 25200 + timezone;

        #elif defined (_32BIT_VERSION_)                  /* Win32 version */
           gz_header.time = gz_header.time - 36000 + timezone;
	#endif

        #ifndef _32BIT_VERSION_                         /* DOS version */
                if(timezone==18000) gz_header.time -= 10800;
                else gz_header.time= gz_header.time - 25200 + timezone;
        #endif

   #endif

	unixtodos(gz_header.time,&filedate,&filetime);
	printf("Archive %s created %02i-%02i-%02i %02i:%02i on ",
	orig_name, filedate.da_day, filedate.da_mon,
	filedate.da_year, filetime.ti_hour, filetime.ti_min);

        switch(gz_header.os){
		case 0x00:
			printf("MS-DOS");
			break;
		case 0x01:
			printf("Amiga");
			break;
		case 0x02:
			printf("VAX/VMS");
			break;
		case 0x03:
			printf("UNIX");
			break;
		case 0x05:
			printf("Atari");
			break;
		case 0x06:
			printf("OS/2");
			break;
		case 0x07:
			printf("Macintosh");
			break;
		case 0x0a:
			printf("TOPS20");
			break;
		case 0x0b:
			printf("Windows NT");
			break;
		case 0x0f:
			printf("PRIMOS");
			break;
		default:
                        printf("unknown (0x%X)",gz_header.os);
		}
	printf(" platform\n");
	updcrc(NULL,0);	/* Initialize CRC engine */
	return 0;
}







int read_pkzip_header(void){

	char orig_name[260]="";/* 260 bytes hopefully is enough for filename */
	char extra_field[260]="";
	fread(&pk_header,sizeof(pk_header),1,in);

	if(pk_header.magic!=0x04034b50)  return 1;

	if(pk_header.bitflag & PK_IS_ENCRYPT){
		fprintf(stderr, PROG_NAME ": error - zip file is encrypted.\n");
		return -1;
		}

	if(pk_header.method!=8) {
		printf(PROG_NAME ": error - ");
		switch(pk_header.method){
			case 0:
			 printf("unstore");
			 break;
			case 1:
			 printf("unshrink");
			 break;
			case 2:
			case 3:
			case 4:
			case 5:
			 printf("unreduce");
			 break;
			case 6:
			 printf("exploding");
			 break;
			case 7:
			 printf("untokenizing");
			 break;
			default:
			 printf("zip compression type %i",pk_header.method);
			 break;
			 }
		printf(" not supported, use unzip to extract\n");
		return -1;
	       }

		fread(orig_name,pk_header.filename_size,1,in);
		fread(extra_field,pk_header.extra_field_length,1,in);

	printf("ZIP Archive containing %s created on ",	orig_name);

	switch(pk_header.os){
		case 0x00:
			printf("MS-DOS");
			break;
		case 0x01:
			printf("Amiga");
			break;
		case 0x02:
			printf("VAX/VMS");
			break;
		case 0x03:
			printf("UNIX");
			break;
		case 0x05:
			printf("Atari");
			break;
		case 0x06:
			printf("OS/2");
			break;
		case 0x07:
			printf("Macintosh");
			break;
		case 0x08:
			printf("Z-System");
			break;
		case 0x09:
			printf("CP/M");
			break;
		case 0x0a:
			printf("TOPS20");
			break;
		case 0x0b:
			printf("Windows NT");
			break;
		case 0x0f:
			printf("PRIMOS");
			break;
		default:
			printf("unknown (0x%X)",pk_header.os);
		}
	printf(" platform");

	if(pk_header.bitflag & PK_IS_ENDHEADER){
		printf(" (Tar-style)");
		headertype=HEADER_PKZIP_END;
		}
	else headertype=HEADER_PKZIP;
        printf("\n");

        if(pk_header.bitflag & PK_IS_EXTENDED){
                printf(PROG_NAME ": warning -  %u bytes extra field ignored\n",
                       pk_header.extra_field_length);
		headertype=HEADER_PKZIP_END;
		}


	updcrc(NULL,0);	/* Initialize CRC engine */
	return 0;
}






int ReadByte(UWORD *x){
	register int ch=getc(in);
	if(ch==EOF)return 0;
	*x=(UWORD)ch;
	return 8;
	}

int FlushOutput(void){
 if (outcnt)
  {
/*  fwrite(outbuf,1,outcnt,stdout);
    printf(outbuf);exit(1); */
    updcrc(outbuf,outcnt);
    switch(command){
	case CMD_EXTRACT:
	    untarstream(outbuf,outcnt);
	    break;
	default:
	    printf(PROG_NAME ": internal error - nothing to do.\n");
	}
    outsiz += outcnt;
    outptr = outbuf;
    outcnt = 0;
  }
  return 0;
	}

